@@ -1,16 +1,19 @@
 #![feature(test)]
 
 extern crate test;
-use solana_ledger::shred::Shred;
-use solana_ledger::shred::SIZE_OF_DATA_SHRED_PAYLOAD;
-use solana_ledger::sigverify_shreds::{
-    sign_shreds_cpu, sign_shreds_gpu, sign_shreds_gpu_pinned_keypair,
+use {
+    solana_ledger::{
+        shred::{Shred, SIZE_OF_DATA_SHRED_PAYLOAD},
+        sigverify_shreds::{sign_shreds_cpu, sign_shreds_gpu, sign_shreds_gpu_pinned_keypair},
+    },
+    solana_perf::{
+        packet::{Packet, Packets},
+        recycler_cache::RecyclerCache,
+    },
+    solana_sdk::signature::Keypair,
+    std::sync::Arc,
+    test::Bencher,
 };
-use solana_perf::packet::{Packet, Packets};
-use solana_perf::recycler_cache::RecyclerCache;
-use solana_sdk::signature::Keypair;
-use std::sync::Arc;
-use test::Bencher;
 
 const NUM_PACKETS: usize = 256;
 const NUM_BATCHES: usize = 1;
